"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.API = void 0;
const tslib_1 = require("tslib");
const __1 = require("..");
const API_worker_1 = require("./API.worker");
/**
 * Class representing an API client for making requests to the Telegram Bot API.
 */
class API extends API_worker_1.ApiWorker {
    /**
     * Use this method to receive incoming updates using long polling.
     * Returns an array of Update objects.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getupdates)
     */
    getUpdates(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.call('getUpdates', params);
            return ((data === null || data === void 0 ? void 0 : data.map((update) => __1.ContextManager.getContext('Update', { client: this.client.clone(), source: update }))) ||
                []);
        });
    }
    /**
     * Use this method to specify a URL and receive incoming updates via an outgoing webhook.
     * Whenever there is an update for the bot, Telegram will send an HTTPS POST request to the specified URL,
     * containing a JSON-serialized Update. In case of an unsuccessful request, Telegram will give up after a reasonable amount of attempts.
     *
     * If you'd like to make sure that the webhook was set by you, you can specify secret data in the parameter secret_token.
     * If specified, the request will contain a header “X-Telegram-Bot-Api-Secret-Token” with the secret token as content.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setwebhook)
     */
    setWebhook(params) {
        return this.call('setWebhook', params);
    }
    /**
     * Use this method to remove webhook integration if you decide to switch back to getUpdates.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deletewebhook)
     */
    deleteWebhook(params) {
        return this.call('deleteWebhook', params);
    }
    /**
     * Use this method to get current webhook status.
     * If the bot is using getUpdates, will return an object with the url field empty.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getwebhookinfo)
     */
    getWebhookInfo() {
        return this.call('getWebhookInfo');
    }
    /**
     * A simple method for testing your bot's authentication token.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getme)
     */
    getMe() {
        return this.getContext('getMe', {}, 'Bot');
    }
    /**
     * Use this method to log out from the cloud Bot API server before launching the bot locally.
     * You must log out the bot before running it locally, otherwise there is no guarantee that
     * the bot will receive updates. After a successful call, you can immediately log in on a
     * local server, but will not be able to log in back to the cloud Bot API server for 10 minutes.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#logout)
     */
    logOut() {
        return this.call('logOut');
    }
    /**
     * Use this method to close the bot instance before moving it from one local server to another.
     * You need to delete the webhook before calling this method to ensure that the bot isn't launched again after server restart.
     * The method will return error 429 in the first 10 minutes after the bot is launched.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#close)
     */
    close() {
        return this.call('close');
    }
    /**
     * Use this method to send text messages.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendmessage)
     */
    sendMessage(params) {
        return this.getContext('sendMessage', params, 'IncomingMessage');
    }
    /**
     * Use this method to forward messages of any kind.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#forwardmessage)
     */
    forwardMessage(params) {
        return this.getContext('forwardMessage', params, 'IncomingMessage');
    }
    /**
     * Use this method to forward multiple messages of any kind.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#forwardmessages)
     */
    forwardMessages(params) {
        // Since an array of MessageId is returned on success, there's no need for getContext
        return this.call('forwardMessages', params);
    }
    /**
     * Use this method to copy messages of any kind.
     * Service messages, giveaway messages, giveaway winners messages, and invoice messages can't be copied.
     * A quiz poll can be copied only if the value of the field correct_option_id is known to the bot.
     * The method is analogous to the method forwardMessage, but the copied message doesn't have a link to the original message.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#copymessage)
     */
    copyMessage(params) {
        // Since a MessageId is returned on success, there's no need for getContext
        return this.call('copyMessage', params);
    }
    /**
     * Use this method to copy messages of any kind.
     * If some of the specified messages can't be found or copied, they are skipped.
     * Service messages, giveaway messages, giveaway winners messages, and invoice messages can't be copied.
     * A quiz poll can be copied only if the value of the field correct_option_id is known to the bot.
     * The method is analogous to the method forwardMessages, but the copied messages don't have a link to the original message.
     * Album grouping is kept for copied messages.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#copymessages)
     */
    copyMessages(params) {
        // Since an array of MessageId is returned on success, there's no need for getContext
        return this.call('copyMessages', params);
    }
    /**
     * Use this method to send photos.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendphoto)
     */
    sendPhoto(params) {
        return this.getContext('sendPhoto', params, 'IncomingMessage');
    }
    /**
     * Use this method to send audio files, if you want Telegram clients to display them in the music player.
     * Your audio must be in the .MP3 or .M4A format.
     * Bots can currently send audio files of up to 50 MB in size, this limit may be changed in the future.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendaudio)
     */
    sendAudio(params) {
        return this.getContext('sendAudio', params, 'IncomingMessage');
    }
    /**
     * Use this method to send general files.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#senddocument)
     */
    sendDocument(params) {
        return this.getContext('sendDocument', params, 'IncomingMessage');
    }
    /**
     * Use this method to send video files, Telegram clients support MPEG4 videos (other formats may be sent as Document).
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendvideo)
     */
    sendVideo(params) {
        return this.getContext('sendVideo', params, 'IncomingMessage');
    }
    /**
     * Use this method to send animation files (GIF or H.264/MPEG-4 AVC video without sound).
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendanimation)
     */
    sendAnimation(params) {
        return this.getContext('sendAnimation', params, 'IncomingMessage');
    }
    /**
     * Use this method to send audio files, if you want Telegram clients to display the file as a playable voice message.
     * For this to work, your audio must be in an .OGG file encoded with OPUS (other formats may be sent as Audio or Document).
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendvoice)
     */
    sendVoice(params) {
        return this.getContext('sendVoice', params, 'IncomingMessage');
    }
    /**
     * Use this method to send video messages.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendvideonote)
     */
    sendVideoNote(params) {
        return this.getContext('sendVideoNote', params, 'IncomingMessage');
    }
    /**
     * Use this method to send a group of photos, videos, documents, or audios as an album.
     * Documents and audio files can be only grouped in an album with messages of the same type.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendmediagroup)
     */
    sendMediaGroup(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.call('sendMediaGroup', params);
            return data.map((update) => __1.ContextManager.getContext('Message', { client: this.client, source: update }));
        });
    }
    /**
     * Use this method to send a point on the map.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendlocation)
     */
    sendLocation(params) {
        return this.getContext('sendLocation', params, 'IncomingMessage');
    }
    /**
     * Use this method to send information about a venue.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendvenue)
     */
    sendVenue(params) {
        return this.getContext('sendVenue', params, 'IncomingMessage');
    }
    /**
     * Use this method to send phone contacts.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendcontact)
     */
    sendContact(params) {
        return this.getContext('sendContact', params, 'IncomingMessage');
    }
    /**
     * Use this method to send a native poll.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendpoll)
     */
    sendPoll(params) {
        return this.getContext('sendPoll', params, 'IncomingMessage');
    }
    /**
     * Use this method to send an animated emoji that will display a random value.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#senddice)
     */
    sendDice(params) {
        return this.getContext('sendDice', params, 'IncomingMessage');
    }
    /**
     * Use this method when you need to tell the user that something is happening on the bot's side.
     * The status is set for 5 seconds or less (when a message arrives from your bot, Telegram clients clear its typing status).
     *
     * We only recommend using this method when a response from the bot will take a noticeable amount of time to arrive.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendchataction)
     */
    sendChatAction(params) {
        return this.call('sendChatAction', params);
    }
    /**
     * Use this method to change the chosen reactions on a message.
     * Service messages can't be reacted to.
     * Automatically forwarded messages from a channel to its discussion group have the same available reactions as messages in the channel.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setmessagecontent)
     */
    setMessageReaction(params) {
        return this.call('setMessageReaction', params);
    }
    /**
     * Use this method to get a list of profile pictures for a user.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getuserprofilephotos)
     */
    getUserProfilePhotos(params) {
        return this.call('getUserProfilePhotos', params);
    }
    /**
     * Use this method to get basic information about a file and prepare it for downloading.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getfile)
     */
    getFile(params) {
        return this.call('getFile', params);
    }
    /**
     * Use this method to ban a user in a group, a supergroup, or a channel.
     * In the case of supergroups and channels, the user will not be able to return to the chat on their own using invite links, etc., unless unbanned first.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#banchatmember)
     */
    banChatMember(params) {
        return this.call('banChatMember', params);
    }
    /**
     * Use this method to unban a previously banned user in a supergroup or channel.
     * The user will not return to the group or channel automatically, but will be able to join via link, etc.
     * The bot must be an administrator for this to work.
     * By default, this method guarantees that after the call the user is not a member of the chat, but will be able to join it.
     * So if the user is a member of the chat they will also be removed from the chat. If you don't want this, use the parameter only_if_banned.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#unbanchatmember)
     */
    unbanChatMember(params) {
        return this.call('unbanChatMember', params);
    }
    /**
     * Use this method to restrict a user in a supergroup.
     * The bot must be an administrator in the supergroup for this to work and must have the appropriate administrator rights.
     * Pass True for all permissions to lift restrictions from a user.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#restrictchatmember)
     */
    restrictChatMember(params) {
        return this.call('restrictChatMember', params);
    }
    /**
     * Use this method to promote or demote a user in a supergroup or a channel.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     * Pass False for all boolean parameters to demote a user.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#promotechatmember)
     */
    promoteChatMember(params) {
        return this.call('promoteChatMember', params);
    }
    /**
     * Use this method to set a custom title for an administrator in a supergroup promoted by the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setchatadministratorcustomtitle)
     */
    setChatAdministratorCustomTitle(params) {
        return this.call('setChatAdministratorCustomTitle', params);
    }
    /**
     * Use this method to ban a channel chat in a supergroup or a channel.
     * Until the chat is unbanned, the owner of the banned chat won't be able to send messages on behalf of any of their channels.
     * The bot must be an administrator in the supergroup or channel for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#banchatsenderchat)
     */
    banChatSenderChat(params) {
        return this.call('banChatSenderChat', params);
    }
    /**
     * Use this method to unban a previously banned channel chat in a supergroup or channel.
     * The bot must be an administrator for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#unbanchatsenderchat)
     */
    unbanChatSenderChat(params) {
        return this.call('unbanChatSenderChat', params);
    }
    /**
     * Use this method to set default chat permissions for all members.
     * The bot must be an administrator in the group or a supergroup for this to work and must have the can_restrict_members administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setchatpermissions)
     */
    setChatPermissions(params) {
        return this.call('setChatPermissions', params);
    }
    /**
     * Use this method to generate a new primary invite link for a chat; any previously generated primary link is revoked.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#exportchatinvitelink)
     */
    exportChatInviteLink(params) {
        return this.call('exportChatInviteLink', params);
    }
    /**
     * Use this method to create an additional invite link for a chat.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     * The link can be revoked using the method revokeChatInviteLink.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#createchatinvitelink)
     */
    createChatInviteLink(params) {
        return this.call('createChatInviteLink', params);
    }
    /**
     * Use this method to edit a non-primary invite link created by the bot.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#editchatinvitelink)
     */
    editChatInviteLink(params) {
        return this.call('editChatInviteLink', params);
    }
    /**
     * Use this method to revoke an invite link created by the bot.
     * If the primary link is revoked, a new link is automatically generated.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#revokechatinvitelink)
     */
    revokeChatInviteLink(params) {
        return this.call('revokeChatInviteLink', params);
    }
    /**
     * Use this method to approve a chat join request.
     * The bot must be an administrator in the chat for this to work and must have the can_invite_users administrator right.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#approvechatjoinrequest)
     */
    approveChatJoinRequest(params) {
        return this.call('approveChatJoinRequest', params);
    }
    /**
     * Use this method to decline a chat join request.
     * The bot must be an administrator in the chat for this to work and must have the can_invite_users administrator right.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#declinechatjoinrequest)
     */
    declineChatJoinRequest(params) {
        return this.call('declineChatJoinRequest', params);
    }
    /**
     * Use this method to set a new profile photo for the chat.
     * Photos can't be changed for private chats.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setchatphoto)
     */
    setChatPhoto(params) {
        return this.call('setChatPhoto', params);
    }
    /**
     * Use this method to delete a chat photo.
     * Photos can't be changed for private chats.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deletechatphoto)
     */
    deleteChatPhoto(params) {
        return this.call('deleteChatPhoto', params);
    }
    /**
     * Use this method to change the title of a chat.
     * Titles can't be changed for private chats.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setchattitle)
     */
    setChatTitle(params) {
        return this.call('setChatTitle', params);
    }
    /**
     * Use this method to change the description of a group, a supergroup, or a channel.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setchatdescription)
     */
    setChatDescription(params) {
        return this.call('setChatDescription', params);
    }
    /**
     * Use this method to add a message to the list of pinned messages in a chat.
     * If the chat is not a private chat, the bot must be an administrator in the chat for this to work and must
     * have the 'can_pin_messages' administrator right in a supergroup or 'can_edit_messages' administrator right in a channel.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#pinchatmessage)
     */
    pinChatMessage(params) {
        return this.call('pinChatMessage', params);
    }
    /**
     * Use this method to remove a message from the list of pinned messages in a chat.
     * If the chat is not a private chat, the bot must be an administrator in the chat for this to work and must
     * have the 'can_pin_messages' administrator right in a supergroup or 'can_edit_messages' administrator right in a channel.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#unpinchatmessage)
     */
    unpinChatMessage(params) {
        return this.call('unpinChatMessage', params);
    }
    /**
     * Use this method to clear the list of pinned messages in a chat.
     * If the chat is not a private chat, the bot must be an administrator in the chat for this to work and must
     * have the 'can_pin_messages' administrator right in a supergroup or 'can_edit_messages' administrator right in a channel.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#unpinallchatmessages)
     */
    unpinAllChatMessages(params) {
        return this.call('unpinAllChatMessages', params);
    }
    /**
     * Use this method for your bot to leave a group, supergroup, or channel. Returns True on success.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#leavechat)
     */
    leaveChat(params) {
        return this.call('leaveChat', params);
    }
    /**
     * Use this method to get up-to-date information about the chat. Returns a Chat object on success.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getchat)
     */
    getChat(params) {
        return this.call('getChat', params);
    }
    /**
     * Use this method to get a list of administrators in a chat, which aren't bots.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getchatadministrators)
     */
    getChatAdministrators(params) {
        return this.call('getChatAdministrators', params);
    }
    /**
     * Use this method to get the number of members in a chat.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getchatmembercount)
     */
    getChatMemberCount(params) {
        return this.call('getChatMemberCount', params);
    }
    /**
     * Use this method to get information about a member of a chat.
     * The method is only guaranteed to work for other users if the bot is an administrator in the chat.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getchatmember)
     */
    getChatMember(params) {
        return this.call('getChatMember', params);
    }
    /**
     * Use this method to set a new group sticker set for a supergroup.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     * Use the field can_set_sticker_set optionally returned in getChat requests to check if the bot can use this method.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setchatstickerset)
     */
    setChatStickerSet(params) {
        return this.call('setChatStickerSet', params);
    }
    /**
     * Use this method to delete a group sticker set from a supergroup.
     * The bot must be an administrator in the chat for this to work and must have the appropriate administrator rights.
     * Use the field can_set_sticker_set optionally returned in getChat requests to check if the bot can use this method.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deletechatstickerset)
     */
    deleteChatStickerSet(params) {
        return this.call('deleteChatStickerSet', params);
    }
    /**
     * Use this method to get custom emoji stickers, which can be used as a forum topic icon by any user.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getforumtopiciconstickers)
     */
    getForumTopicIconStickers() {
        return this.call('getForumTopicIconStickers');
    }
    /**
     * Use this method to create a topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have the can_manage_topics administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#createforumtopic)
     */
    createForumTopic(params) {
        return this.call('createForumTopic', params);
    }
    /**
     * Use this method to edit name and icon of a topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have can_manage_topics administrator rights,
     * unless it is the creator of the topic.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#editforumtopic)
     */
    editForumTopic(params) {
        return this.call('editForumTopic', params);
    }
    /**
     * Use this method to close an open topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have the can_manage_topics administrator rights,
     * unless it is the creator of the topic.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#closeforumtopic)
     */
    closeForumTopic(params) {
        return this.call('closeForumTopic', params);
    }
    /**
     * Use this method to reopen a closed topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have the can_manage_topics administrator rights,
     * unless it is the creator of the topic.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#reopenforumtopic)
     */
    reopenForumTopic(params) {
        return this.call('reopenForumTopic', params);
    }
    /**
     * Use this method to delete a forum topic along with all its messages in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have the can_delete_messages administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deleteforumtopic)
     */
    deleteForumTopic(params) {
        return this.call('deleteForumTopic', params);
    }
    /**
     * Use this method to clear the list of pinned messages in a forum topic.
     * The bot must be an administrator in the chat for this to work and must have the can_pin_messages administrator right in the supergroup.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#unpinallforumtopicmessages)
     */
    unpinAllForumTopicMessages(params) {
        return this.call('unpinAllForumTopicMessages', params);
    }
    /**
     * Use this method to edit the name of the 'General' topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have can_manage_topics administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#editgeneralforumtopic)
     */
    editGeneralForumTopic(params) {
        return this.call('editGeneralForumTopic', params);
    }
    /**
     * Use this method to close an open 'General' topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have the can_manage_topics administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#closegeneralforumtopic)
     */
    closeGeneralForumTopic(params) {
        return this.call('closeGeneralForumTopic', params);
    }
    /**
     * Use this method to reopen a closed 'General' topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have the can_manage_topics administrator rights.
     * The topic will be automatically unhidden if it was hidden.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#reopengeneralforumtopic)
     */
    reopenGeneralForumTopic(params) {
        return this.call('reopenGeneralForumTopic', params);
    }
    /**
     * Use this method to hide the 'General' topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have the can_manage_topics administrator rights.
     * The topic will be automatically closed if it was open.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#hidegeneralforumtopic)
     */
    hideGeneralForumTopic(params) {
        return this.call('hideGeneralForumTopic', params);
    }
    /**
     * Use this method to unhide the 'General' topic in a forum supergroup chat.
     * The bot must be an administrator in the chat for this to work and must have the can_manage_topics administrator rights.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#unhidegeneralforumtopic)
     */
    unhideGeneralForumTopic(params) {
        return this.call('unhideGeneralForumTopic', params);
    }
    /**
     * Use this method to clear the list of pinned messages in a General forum topic.
     * The bot must be an administrator in the chat for this to work and must have the can_pin_messages administrator right in the supergroup.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#unpinallgeneralforumtopicmessages)
     */
    unpinAllGeneralForumTopicMessages(params) {
        return this.call('unpinAllGeneralForumTopicMessages', params);
    }
    /**
     * Use this method to send answers to callback queries sent from inline keyboards.
     * The answer will be displayed to the user as a notification at the top of the chat screen or as an alert.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#answercallbackquery)
     */
    answerCallbackQuery(params) {
        return this.call('answerCallbackQuery', params);
    }
    /**
     * Use this method to get the list of boosts added to a chat by a user.
     * Requires administrator rights in the chat.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getuserchatboosts)
     */
    getUserChatBoosts(params) {
        return this.call('getUserChatBoosts', params);
    }
    /**
     * Use this method to get information about the connection of the bot with a business account.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getbusinessconnection)
     */
    getBusinessConnection() {
        return this.getContext('getBusinessConnection', {}, 'BusinessConnection');
    }
    /**
     * Use this method to change the list of the bot's commands.
     * See this manual for more details about bot commands.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setmycommands)
     */
    setMyCommands(params) {
        return this.call('setMyCommands', params);
    }
    /**
     * Use this method to delete the list of the bot's commands
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deletemycommands)
     */
    deleteMyCommands(params) {
        return this.call('deleteMyCommands', params);
    }
    /**
     * Use this method to get the current list of the bot's commands for the given scope and user language.
     * If commands aren't set, an empty list is returned.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getmycommands)
     */
    getMyCommands(params) {
        return this.call('getMyCommands', params);
    }
    /**
     * Use this method to change the bot's name.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setmyname)
     */
    setMyName(params) {
        return this.call('setMyName', params);
    }
    /**
     * Use this method to get the current bot name for the given user language.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getmyname)
     */
    getMyName(params) {
        return this.call('getMyName', params);
    }
    /**
     * Use this method to change the bot's description, which is shown in the chat with the bot if the chat is empty.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setmydescription)
     */
    setMyDescription(params) {
        return this.call('setMyDescription', params);
    }
    /**
     * Use this method to get the current bot description for the given user language.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getmydescription)
     */
    getMyDescription(params) {
        return this.call('getMyDescription', params);
    }
    /**
     * Use this method to change the bot's short description, which is shown on the bot's profile page
     * and is sent together with the link when users share the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setmyshortdescription)
     */
    setMyShortDescription(params) {
        return this.call('setMyShortDescription', params);
    }
    /**
     * Use this method to get the current bot short description for the given user language.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getmyshortdescription)
     */
    getMyShortDescription(params) {
        return this.call('getMyShortDescription', params);
    }
    /**
     * Use this method to change the bot's menu button in a private chat, or the default menu button.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setchatmenubutton)
     */
    setChatMenuButton(params) {
        return this.call('setChatMenuButton', params);
    }
    /**
     * Use this method to get the current value of the bot's menu button in a private chat, or the default menu button.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getchatmenubutton)
     */
    getChatMenuButton(params) {
        return this.call('getChatMenuButton', params);
    }
    /**
     * Use this method to change the default administrator rights requested by the bot
     * when it's added as an administrator to groups or channels.
     * These rights will be suggested to users, but they are free to modify the list before adding the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setmydefaultadministratorrights)
     */
    setMyDefaultAdministratorRights(params) {
        return this.call('setMyDefaultAdministratorRights', params);
    }
    /**
     * Use this method to get the current default administrator rights of the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getmydefaultadministratorrights)
     */
    getMyDefaultAdministratorRights(params) {
        return this.call('getMyDefaultAdministratorRights', params);
    }
    /**
     * Use this method to edit text and game messages.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#editmessagetext)
     */
    editMessageText(params) {
        return this.getContext('editMessageText', params, 'IncomingMessage');
    }
    /**
     * Use this method to edit captions of messages.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#editmessagecaption)
     */
    editMessageCaption(params) {
        return this.getContext('editMessageCaption', params, 'IncomingMessage');
    }
    /**
     * Use this method to edit animation, audio, document, photo, or video messages.
     * If a message is part of a message album, then it can be edited only to an audio for audio albums,
     * only to a document for document albums and to a photo or a video otherwise.
     * When an inline message is edited, a new file can't be uploaded; use a previously uploaded file via its file_id or specify a URL.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#editmessagemedia)
     */
    editMessageMedia(params) {
        return this.getContext('editMessageMedia', params, 'IncomingMessage');
    }
    /**
     * Use this method to edit live location messages.
     * A location can be edited until its live_period expires or editing is explicitly disabled by a call to stopMessageLiveLocation.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#editmessagelivelocation)
     */
    editMessageLiveLocation(params) {
        return this.getContext('editMessageLiveLocation', params, 'IncomingMessage');
    }
    /**
     * Use this method to stop updating a live location message before live_period expires.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#stopmessagelivelocation)
     */
    stopMessageLiveLocation(params) {
        return this.getContext('stopMessageLiveLocation', params, 'IncomingMessage');
    }
    /**
     * Use this method to edit only the reply markup of messages.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#editmessagereplymarkup)
     */
    editMessageReplyMarkup(params) {
        return this.getContext('editMessageReplyMarkup', params, 'IncomingMessage');
    }
    /**
     * Use this method to stop a poll which was sent by the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#stoppoll)
     */
    stopPoll(params) {
        return this.getContext('stopPoll', params, 'PollContext');
    }
    /**
     * Use this method to delete a message, including service messages, with the following limitations:
     * - A message can only be deleted if it was sent less than 48 hours ago.
     * - Service messages about a supergroup, channel, or forum topic creation can't be deleted.
     * - A dice message in a private chat can only be deleted if it was sent more than 24 hours ago.
     * - Bots can delete outgoing messages in private chats, groups, and supergroups.
     * - Bots can delete incoming messages in private chats.
     * - Bots granted can_post_messages permissions can delete outgoing messages in channels.
     * - If the bot is an administrator of a group, it can delete any message there.
     * - If the bot has can_delete_messages permission in a supergroup or a channel, it can delete any message there.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deletemessage)
     */
    deleteMessage(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.client.database.message.deleteMessages([params.message_id]);
            return this.call('deleteMessage', params);
        });
    }
    /**
     * Use this method to delete multiple messages simultaneously.
     * If some of the specified messages can't be found, they are skipped.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deletemessages)
     */
    deleteMessages(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.client.database.message.deleteMessages(params.message_ids);
            return this.call('deleteMessages', params);
        });
    }
    /**
     * Use this method to send static .WEBP, animated .TGS, or video .WEBM stickers.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendsticker)
     */
    sendSticker(params) {
        return this.getContext('sendSticker', params, 'IncomingMessage');
    }
    /**
     * Use this method to get a sticker set.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getstickerset)
     */
    getStickerSet(params) {
        return this.call('getStickerSet', params);
    }
    /**
     * Use this method to get information about custom emoji stickers by their identifiers.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getcustomemojistickers)
     */
    getCustomEmojiStickers(params) {
        return this.call('getCustomEmojiStickers', params);
    }
    /**
     * Use this method to upload a file with a sticker for later use in the createNewStickerSet, addStickerToSet, or replaceStickerInSet methods
     * (the file can be used multiple times). Returns the uploaded File on success.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#uploadstickerfile)
     */
    uploadStickerFile(params) {
        return this.call('uploadStickerFile', params);
    }
    /**
     * Use this method to create a new sticker set owned by a user.
     * The bot will be able to edit the sticker set thus created.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#createnewstickerset)
     */
    createNewStickerSet(params) {
        return this.call('createNewStickerSet', params);
    }
    /**
     * Use this method to add a new sticker to a set created by the bot.
     * Emoji sticker sets can have up to 200 stickers. Other sticker sets can have up to 120 stickers.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#addstickertoset)
     */
    addStickerToSet(params) {
        return this.call('addStickerToSet', params);
    }
    /**
     * Use this method to move a sticker in a set created by the bot to a specific position.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setstickerpositioninset)
     */
    setStickerPositionInSet(params) {
        return this.call('setStickerPositionInSet', params);
    }
    /**
     * Use this method to delete a sticker from a set created by the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deletestickerfromset)
     */
    deleteStickerFromSet(params) {
        return this.call('deleteStickerFromSet', params);
    }
    /**
     * Use this method to replace an existing sticker in a sticker set with a new one.
     * The method is equivalent to calling deleteStickerFromSet, then addStickerToSet, then setStickerPositionInSet.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#replacestickerinset)
     */
    replaceStickerInSet(params) {
        return this.call('replaceStickerInSet', params);
    }
    /**
     * Use this method to change the list of emoji assigned to a regular or custom emoji sticker.
     * The sticker must belong to a sticker set created by the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setstickeremojilist)
     */
    setStickerEmojiList(params) {
        return this.call('setStickerEmojiList', params);
    }
    /**
     * Use this method to change search keywords assigned to a regular or custom emoji sticker.
     * The sticker must belong to a sticker set created by the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setstickerkeywords)
     */
    setStickerKeywords(params) {
        return this.call('setStickerKeywords', params);
    }
    /**
     * Use this method to change the mask position of a mask sticker.
     * The sticker must belong to a sticker set that was created by the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setstickermaskposition)
     */
    setStickerMaskPosition(params) {
        return this.call('setStickerMaskPosition', params);
    }
    /**
     * Use this method to set the title of a created sticker set.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setstickersettitle)
     */
    setStickerSetTitle(params) {
        return this.call('setStickerSetTitle', params);
    }
    /**
     * Use this method to set the thumbnail of a regular or mask sticker set.
     * The format of the thumbnail file must match the format of the stickers in the set.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setstickersetthumbnail)
     */
    setStickerSetThumbnail(params) {
        return this.call('setStickerSetThumbnail', params);
    }
    /**
     * Use this method to set the thumbnail of a custom emoji sticker set.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setcustomemojistickersetthumbnail)
     */
    setCustomEmojiStickerSetThumbnail(params) {
        return this.call('setCustomEmojiStickerSetThumbnail', params);
    }
    /**
     * Use this method to delete a sticker set that was created by the bot.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#deletestickerset)
     */
    deleteStickerSet(params) {
        return this.call('deleteStickerSet', params);
    }
    /**
     * Use this method to set the result of an interaction with a Web App
     * and send a corresponding message on behalf of the user to the chat from which the query originated.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#answerwebappquery)
     */
    answerWebAppQuery(params) {
        return this.call('answerWebAppQuery', params);
    }
    /**
     * Use this method to send invoices.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendinvoice)
     */
    sendInvoice(params) {
        return this.getContext('sendInvoice', params, 'IncomingMessage');
    }
    /**
     * Use this method to create a link for an invoice.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#createinvoicelink)
     */
    createInvoiceLink(params) {
        return this.call('createInvoiceLink', params);
    }
    /**
     * If you sent an invoice requesting a shipping address and the parameter is_flexible was specified,
     * the Bot API will send an Update with a shipping_query field to the bot.
     * Use this method to reply to shipping queries.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#answershippingquery)
     */
    answerShippingQuery(params) {
        return this.call('answerShippingQuery', params);
    }
    /**
     * Once the user has confirmed their payment and shipping details,
     * the Bot API sends the final confirmation in the form of an Update with the field pre_checkout_query.
     * Use this method to respond to such pre-checkout queries.
     * Note: The Bot API must receive an answer within 10 seconds after the pre-checkout query was sent.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#answerprecheckoutquery)
     */
    answerPreCheckoutQuery(params) {
        return this.call('answerPreCheckoutQuery', params);
    }
    /**
     * Sends answers to an inline query.
     * Use this method to send answers to an inline query. On success, True is returned.
     * No more than 50 results per query are allowed.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#answerinlinequery)
     */
    answerInlineQuery(params) {
        return this.call('answerInlineQuery', params);
    }
    /**
     * Use this method to send a game.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendgame)
     */
    sendGame(params) {
        return this.getContext('sendGame', params, 'IncomingMessage');
    }
    /**
     * Use this method to set the score of the specified user in a game message.
     * On success, if the message is not an inline message, the Message is returned, otherwise True is returned.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#setgamescore)
     */
    setGameScore(params) {
        return this.getContext('setGameScore', params, 'IncomingMessage');
    }
    /**
     * Use this method to get data for high score tables.
     * Will return the score of the specified user and several of their neighbors in a game.
     *
     * This method will currently return scores for the target user, plus two of their closest neighbors on each side.
     * Will also return the top three users if the user and their neighbors are not among them.
     * Please note that this behavior is subject to change.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getgamehighscores)
     */
    getGameHighScores(params) {
        return this.call('getGameHighScores', params);
    }
    /**
     * Use this method to refund a star payment.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#refundstarpayment)
     */
    refundStarPayment(params) {
        return this.call('refundStarPayment', params);
    }
    /**
     * Use this method to get star transactions.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#getstartransactions)
     */
    getStarTransactions(params) {
        return this.call('getStarTransactions', params);
    }
    /**
     * Use this method to send paid media.
     *
     * [Telegram Documentation](https://core.telegram.org/bots/api#sendpaidmedia)
     */
    sendPaidMedia(params) {
        return this.getContext('sendPaidMedia', params, 'IncomingMessage');
    }
}
exports.API = API;
