"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiWorker = void 0;
const tslib_1 = require("tslib");
const axios_1 = tslib_1.__importDefault(require("axios"));
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const __1 = require("..");
const keyboard_1 = require("../keyboard");
const APIError_1 = require("./APIError");
const MiddlewareManager_1 = require("./middleware/MiddlewareManager");
class ApiWorker {
    /**
     * Set default parameters for a specific API method.
     * @param {string} method - The API method for which default parameters are being set.
     * @param {Record<string, any>} [params] - Optional default parameters to be set for the specified method.
     */
    setDefaultParams(method, params) {
        if (params) {
            this.defaultParams[method] = params;
        }
    }
    /**
     * Initializes the ApiWorker instance with the provided bot token.
     */
    constructor(client, state = {}) {
        this.client = client;
        this.state = state;
        this.defaultParams = {}; // Private variable to store default parameters for API calls
        this.middlewareManager = new MiddlewareManager_1.MiddlewareManager(this.client);
        // Setting the base URL for API requests by concatenating the Telegram bot token to the API URL
        this.url = 'https://api.telegram.org/bot' + client.params.token;
        // Инициализируем менеджер с middleware из декораторов
    }
    use(middleware) {
        this.middlewareManager.use(middleware);
    }
    isUpload(params) {
        const checkValue = (value) => {
            if ((typeof value === 'string' && node_fs_1.default.existsSync(value)) || Buffer.isBuffer(value)) {
                return true;
            }
            if (value && typeof value === 'object' && !Buffer.isBuffer(value)) {
                return Object.values(value).some((nestedValue) => checkValue(nestedValue));
            }
            return false;
        };
        return Object.values(params).some((value) => checkValue(value));
    }
    /**
     * Convert parameters into FormData object for API requests.
     * This method prepares the parameters to be sent in a POST request as FormData,
     * which is suitable for uploading files or other multipart data.
     * @param {Record<string, any>} params - The parameters to be converted into FormData.
     * @returns {FormData} - The FormData object containing the converted parameters.
     */
    getFormData(params) {
        // Create a new FormData instance
        const formData = new FormData();
        // Iterate through each key-value pair in the parameters object
        for (let [key, value] of Object.entries(params)) {
            if (Buffer.isBuffer(value)) {
                // Append the Buffer to FormData, specifying a default filename "file.data"
                //@ts-ignore
                formData.append(key, new Blob([value]), 'file.data');
            }
            else if (typeof value === 'object') {
                // Serialize the reply_markup object to JSON string
                formData.append(key, JSON.stringify(value));
            }
            else if (typeof value === 'string' && node_fs_1.default.existsSync(value)) {
                // Read the file content, create a Blob, and append it to FormData, specifying a default filename "file.data"
                //@ts-ignore
                formData.append(key, new Blob([node_fs_1.default.readFileSync(value)]), params.filename || 'file.data');
            }
            else {
                // For other types of values, simply append them to FormData
                formData.append(key, value);
            }
        }
        // Return the FormData object containing the converted parameters
        return formData;
    }
    /**
     * Make an API call to the Telegram Bot API.
     * @param {string} method - The API method to be called.
     * @param {Record<string, any>} [params] - Optional parameters to be passed in the API call.
     * @returns {Promise<any>} - A promise resolving to the response data from the API call.
     */
    call(method, params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c;
            let mergedParams = Object.assign(Object.assign({}, this.defaultParams[method]), params);
            const ctx = {
                method,
                params: mergedParams,
                isExecuted: false,
                state: this.state,
            };
            try {
                const result = yield this.middlewareManager.executeBefore(ctx);
                if (!result)
                    return null;
                if ((_a = mergedParams.reply_markup) === null || _a === void 0 ? void 0 : _a.inline_keyboard)
                    mergedParams.reply_markup.inline_keyboard = yield (0, keyboard_1.KeyboardConvert)(this.client, mergedParams.reply_markup.inline_keyboard);
                const response = yield (0, axios_1.default)({
                    method: 'POST',
                    url: `${this.url}/${method}`,
                    data: this.isUpload(result) ? this.getFormData(result) : result,
                });
                ctx.result = response.data.result;
                yield this.middlewareManager.executeAfter(ctx);
                return ctx.result;
            }
            catch (error) {
                yield this.middlewareManager.executeError(ctx, error);
                if (axios_1.default.isAxiosError(error) && !((_b = error.response) === null || _b === void 0 ? void 0 : _b.data))
                    return this.call(method, params);
                if (axios_1.default.isAxiosError(error))
                    throw new APIError_1.APIError({ method, params: ctx.params, error: (_c = error.response) === null || _c === void 0 ? void 0 : _c.data });
                throw error;
            }
        });
    }
    getContext(methodName, params, context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const data = yield this.call(methodName, params);
            return __1.ContextManager.getContext(context, { client: this.client, source: data });
        });
    }
}
exports.ApiWorker = ApiWorker;
