"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.migrateCommand = void 0;
const tslib_1 = require("tslib");
const commander_1 = require("commander");
const fs_1 = require("fs");
const path_1 = require("path");
const ts_node_1 = require("ts-node");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const fs_2 = tslib_1.__importDefault(require("fs"));
(0, ts_node_1.register)(); // Register `ts-node` to handle TypeScript files
// Helper function to recursively collect files
function getAllFiles(dirPath, files = []) {
    const entries = (0, fs_1.readdirSync)(dirPath);
    for (const entry of entries) {
        const fullPath = (0, path_1.join)(dirPath, entry);
        if ((0, fs_1.statSync)(fullPath).isDirectory()) {
            getAllFiles(fullPath, files);
        }
        else if ((0, path_1.extname)(fullPath) === '.ts' && fullPath.endsWith('.override.ts')) {
            files.push(fullPath);
        }
    }
    return files;
}
exports.migrateCommand = new commander_1.Command('migrate')
    .description('Run contexts migration')
    .argument('[path]', 'Optional path to run the migration')
    .action((path) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const migrationPath = path || (0, path_1.join)(process.cwd(), 'src');
    const generatedFilePath = (0, path_1.join)(process.cwd(), 'node_modules/evogram/lib/migrated/index.d.ts');
    const backupFilePath = (0, path_1.join)(process.cwd(), 'node_modules/evogram/lib/migrated/index.backup.d.ts');
    if (!(0, fs_1.existsSync)(migrationPath))
        return console.error(`Error: The directory "${migrationPath}" does not exist.`);
    const files = getAllFiles(migrationPath);
    if (files.length === 0)
        return console.error(`Error: No ".override.ts" files found in the directory "${migrationPath}".`);
    const generatedImports = {};
    const existingImports = new Set();
    // Backup the existing index.d.ts file
    if (!(0, fs_1.existsSync)(backupFilePath)) {
        (0, fs_1.copyFileSync)(generatedFilePath, backupFilePath);
    }
    else {
        (0, fs_1.copyFileSync)(backupFilePath, generatedFilePath);
    }
    // Process each .override.ts file
    for (const filePath of files) {
        try {
            const importedModule = yield Promise.resolve(`${filePath}`).then(s => tslib_1.__importStar(require(s)));
            if (importedModule.default && typeof importedModule.default === 'function') {
                const importedClass = importedModule.default;
                const parentClass = Object.getPrototypeOf(importedClass);
                // Get class name from the filename (remove .override.ts)
                const className = (0, path_1.basename)(filePath, '.override.ts');
                // Store the import statement using the className derived from the filename
                generatedImports[className] = `export { default as ${className} } from "${filePath.replace(/\\/g, '/')}";\n`;
                existingImports.add(className);
            }
            else {
                console.log(`No default class export found in: ${filePath}`);
            }
        }
        catch (err) {
            console.error(`Error importing file: ${filePath}`, err);
        }
    }
    // Create or update the index.d.ts file
    let updatedContent = '';
    if ((0, fs_1.existsSync)(generatedFilePath)) {
        try {
            let indexFileContent = (0, fs_1.readFileSync)(generatedFilePath, 'utf-8');
            const lines = indexFileContent.split('\n');
            updatedContent = lines
                .map((line) => {
                // Extract class name from the import line
                const match = line.match(/export\s+\{\s+(default as )?(\w+)\s+\}/);
                if (match) {
                    const className = match[2];
                    return generatedImports[className] || line;
                }
                return line; // Return unchanged line
            })
                .join('\n');
        }
        catch (err) {
            console.error(`Error reading file: ${generatedFilePath}`, err);
        }
    }
    else {
        updatedContent = Object.values(generatedImports).join('\n');
    }
    // Write the updated content back to the index.d.ts file
    try {
        (0, fs_1.writeFileSync)(generatedFilePath, updatedContent, 'utf-8');
        console.log(`File "${generatedFilePath}" has been updated successfully.`);
    }
    catch (err) {
        console.error(`Error writing file: ${generatedFilePath}`, err);
    }
    // Build index.ts using tsc
    try {
        typescript_1.default.transpileModule(fs_2.default.readFileSync(generatedFilePath, 'utf-8'), {});
    }
    catch (err) {
        console.error('Error during TypeScript build process:', err);
    }
}));
