"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandManager = void 0;
const tslib_1 = require("tslib");
class CommandManager {
    constructor(client) {
        this.client = client;
        /**
         * Instance array to hold loaded Command objects.
         */
        this.commands = [];
    }
    /**
     * Loads commands into the `commands` array for the current instance.
     *
     * If a list of `CommandInstance` constructors is provided, the method creates instances
     * of each command using the associated `Evogram` client.
     * If no list is provided, it uses the commands from the static registry (`CommandManager.commands`).
     *
     * @param {CommandInstance[]} [commands] - An optional array of command constructors to load.
     */
    loadCommands(commands) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (commands) {
                this.commands = commands.map((x) => new x(this.client));
            }
            else {
                this.commands = CommandManager.commands.map((x) => new x(this.client));
            }
            this.commands = this.commands.sort((a, b) => (b.params.priority || 0) - (a.params.priority || 0));
        });
    }
    /** Sets the bot commands for the client. */
    setBotCommands() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            var _a, _b, _c, _d;
            const commands = [];
            // Iterate through all the commands in the CommandManager.
            // If a command has a description...
            for (const command of this.commands.filter((x) => x.params.description)) {
                if (command.params.description) {
                    const descriptions = Array.isArray(command.params.description) ? command.params.description : [command.params.description];
                    let defaultCommandName = ((_a = command.params.name) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || ((_b = command.params.name) === null || _b === void 0 ? void 0 : _b.toString().toLowerCase());
                    // Iterate through each language and text pair in the command's description.
                    // If there's already a command set for the current language...
                    // prettier-ignore
                    for (const { text, language, scope, command: commandName } of descriptions) {
                        const scopes = scope ? (Array.isArray(scope) ? scope : [scope]) : ['default'];
                        for (const scope of scopes)
                            commands.find((x) => { var _a; return x.language_code === language && ((_a = x.scope) === null || _a === void 0 ? void 0 : _a.type) === scope; })
                                ? (_c = commands.find((x) => { var _a; return x.language_code === language && ((_a = x.scope) === null || _a === void 0 ? void 0 : _a.type) === scope; })) === null || _c === void 0 ? void 0 : _c.commands.push({ command: commandName || defaultCommandName, description: text })
                                : commands.find((x) => x.scope && x.scope.type === scope)
                                    ? (_d = commands.find((x) => x.scope && x.scope.type === scope)) === null || _d === void 0 ? void 0 : _d.commands.push({ command: (commandName === null || commandName === void 0 ? void 0 : commandName.toLowerCase()) || defaultCommandName, description: text })
                                    //@ts-ignore
                                    : commands.push({ language_code: language, commands: [{ command: (commandName === null || commandName === void 0 ? void 0 : commandName.toLowerCase()) || defaultCommandName, description: text }], scope: { type: scope } });
                    }
                }
            }
            for (const item of commands)
                yield this.client.api.setMyCommands(item);
        });
    }
    /**
     * Gets a command from the list given a command context.
     * @param {CommandContext} context The command context to search for
     * @returns {(Command | undefined)} Returns the command or undefined if not found
     */
    getCommand(context) {
        for (const command of this.commands)
            if (command.isExecutable(context))
                return command;
    }
}
exports.CommandManager = CommandManager;
/**
 * Static array to store registered commands and their parameters.
 * @static
 */
CommandManager.commands = [];
