"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArgumentParser = void 0;
exports.getCommandArguments = getCommandArguments;
exports.getByNewline = getByNewline;
exports.getBySpace = getBySpace;
exports.getByParameterized = getByParameterized;
const tslib_1 = require("tslib");
const migrated_1 = require("../migrated");
const logger_1 = require("../logger");
class ArgumentParser {
    constructor(context, command) {
        this.logger = new logger_1.Logger('ArgumentParser');
        this.context = context;
        this.command = command;
        this.args = context instanceof migrated_1.CommandContext ? command.arguments : command.inlineArguments;
        this.argsType = (typeof command.params.argsMethod === 'string' ? [command.params.argsMethod] : command.params.argsMethod) || [
            'space',
        ];
        this.savedArgs = this.getSavedArgs();
        this.text = this.getCommandText();
    }
    getSavedArgs() {
        if (this.context instanceof migrated_1.InlineQueryContext) {
            return Object.assign(Object.assign({}, this.context.arguments), ((typeof this.context.state.callbackData === 'object' ? this.context.state.callbackData.payload : {}) || {}));
        }
        return Object.assign({}, ((typeof this.context.state.callbackData === 'object' ? this.context.state.callbackData.payload : {}) || {}));
    }
    getCommandText() {
        var _a, _b, _c, _d, _e;
        let text;
        if (this.context instanceof migrated_1.CommandContext)
            text = (_b = (_a = this.context.message) === null || _a === void 0 ? void 0 : _a.text) === null || _b === void 0 ? void 0 : _b.replace(/^\/(\S+)\s?/, '').trim();
        else if (this.context instanceof migrated_1.InlineQueryContext)
            text = (_d = (_c = this.context) === null || _c === void 0 ? void 0 : _c.query) === null || _d === void 0 ? void 0 : _d.replace(/^\/(\S+)\s?/, '').trim();
        else if (this.context instanceof migrated_1.MessageContext)
            text = (_e = this.context.text) === null || _e === void 0 ? void 0 : _e.replace(/^\/(\S+)\s?/, '').trim();
        for (const alias of this.command.params.aliases || [])
            if (typeof alias === 'string' && (text === null || text === void 0 ? void 0 : text.startsWith(alias)))
                text = text.replace(alias, '').trim();
        return text;
    }
    getMissingArgs() {
        return this.args.filter((arg) => !this.savedArgs[typeof arg === 'string' ? arg : arg.name] && !arg.isOptional);
    }
    validateArguments(resultArguments) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            var _a;
            let validateArgs = {};
            if (this.context instanceof migrated_1.CommandContext)
                for (const arg of this.args) {
                    let isValid = false;
                    let validationError = null;
                    while (!isValid) {
                        const currentValue = resultArguments[arg.name];
                        if (arg.question !== undefined && (currentValue === undefined || validationError)) {
                            resultArguments[arg.name] = yield arg.question({
                                args: resultArguments,
                                validateArgs,
                                context: this.context,
                                value: currentValue,
                                command: this.command,
                                arg,
                                error: validationError,
                            });
                            validationError = null;
                        }
                        if (arg.validators.length) {
                            try {
                                const newValue = yield arg.validators.reduce((promise, validator) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                                    const prevValue = yield promise;
                                    return validator({
                                        args: resultArguments,
                                        validateArgs,
                                        //@ts-ignore
                                        context: this.context,
                                        value: prevValue,
                                        command: this.command,
                                        arg,
                                    });
                                }), Promise.resolve(resultArguments[arg.name]));
                                if ((newValue === undefined || newValue === null) && !arg.isOptional) {
                                    if (arg.question !== undefined && arg.askAgain) {
                                        this.logger.warn(`Argument "${arg.name}" is not valid, asking again...`);
                                        resultArguments[arg.name] = undefined;
                                        continue;
                                    }
                                    this.logger.warn(`Argument "${arg.name}" in command "${this.command.params.name}" is not valid`);
                                    return null;
                                }
                                validateArgs[arg.name] = newValue;
                                isValid = true;
                            }
                            catch (error) {
                                if (arg.question !== undefined) {
                                    validationError = error;
                                    resultArguments[arg.name] = undefined;
                                    continue;
                                }
                                throw error;
                            }
                        }
                        else {
                            validateArgs[arg.name] = currentValue;
                            isValid = true;
                        }
                    }
                }
            else
                validateArgs = resultArguments;
            if (this.args.find((x) => (validateArgs[x.name] === undefined || validateArgs[x.name] === null) && !x.isOptional)) {
                this.logger.warn(`Argument "${(_a = this.args.find((x) => (validateArgs[x.name] === undefined || validateArgs[x.name] === null) && !x.isOptional)) === null || _a === void 0 ? void 0 : _a.name}" in command "${this.command.params.name}" is not specified by user`);
                return null;
            }
            return { resultArguments, validateArgs };
        });
    }
    parse() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.command.arguments && !this.command.inlineArguments)
                return { resultArguments: {}, validateArgs: {} };
            let resultArguments = this.savedArgs;
            const missingArgs = this.getMissingArgs();
            if (!(this.context instanceof migrated_1.InlineQueryContext))
                resultArguments = yield this.parseArguments(missingArgs, resultArguments);
            return this.validateArguments(resultArguments);
        });
    }
    parseArguments(missingArgs, resultArguments) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            var _a;
            for (const type of this.argsType) {
                const splitSpace = ((_a = this.text) === null || _a === void 0 ? void 0 : _a.split(' ')) || [];
                if (this.context instanceof migrated_1.CommandContext ? !this.context.callbackQuery : true) {
                    if (['parameterized', 'space', 'fulltext', 'newline'].includes(type) && !this.text)
                        continue;
                    else if (type === 'space' && this.argsType.includes('parameterized') && this.text && this.text.split('--').length > 1)
                        continue;
                    else if (type === 'space' && missingArgs.length > splitSpace.length)
                        continue;
                    else if (type === 'parameterized' && this.text && this.text.split('--').length < 2)
                        continue;
                }
                resultArguments = yield this.parseByType(type, resultArguments, missingArgs);
            }
            return resultArguments;
        });
    }
    parseByType(type, resultArguments, missingArgs) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.text)
                return resultArguments;
            switch (type) {
                case 'space':
                    const spaceValue = getBySpace(this.text, this.args);
                    if (spaceValue)
                        resultArguments = Object.assign(Object.assign({}, spaceValue), this.savedArgs);
                    break;
                case 'fulltext':
                    const fulltextValue = { [typeof missingArgs[0] === 'string' ? missingArgs[0] : missingArgs[0].name]: this.text };
                    resultArguments = Object.assign(Object.assign({}, this.savedArgs), fulltextValue);
                    break;
                case 'parameterized':
                    const paramValue = getByParameterized(this.text);
                    if (paramValue)
                        resultArguments = Object.assign(Object.assign({}, this.savedArgs), paramValue);
                    break;
                case 'newline':
                    const newlineValue = getByNewline(this.text, this.args);
                    if (newlineValue)
                        resultArguments = Object.assign(Object.assign({}, this.savedArgs), newlineValue);
                    break;
            }
            return resultArguments;
        });
    }
}
exports.ArgumentParser = ArgumentParser;
function getCommandArguments(context, command) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const parser = new ArgumentParser(context, command);
        return parser.parse();
    });
}
function getByNewline(text, args) {
    const splitLines = text.split('\n').filter((x) => x.trim().length > 0);
    return Object.assign(
    //@ts-ignore
    ...args.map((x, i) => ({ [typeof x === 'string' ? x.replace('?', '') : x.name.replace('?', '')]: splitLines[i] })));
}
function getBySpace(text, args) {
    const splitSpace = text.split(' ');
    const result = args.map((x, i) => ({ [x.name]: splitSpace[i] }));
    if (!(result === null || result === void 0 ? void 0 : result.length))
        return null;
    //@ts-ignore
    return Object.assign(...result);
}
function getByParameterized(text) {
    const split = text.split('--');
    if (!split)
        return;
    split.splice(0, 1);
    //@ts-ignore
    return Object.assign(...split.map((x) => { var _a; return ({ [x.split('=')[0]]: (_a = x.split('=')[1]) === null || _a === void 0 ? void 0 : _a.trim() }); }));
}
