"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const typeorm_1 = require("typeorm");
const Message_service_1 = require("../Message.service");
const Message_entity_1 = require("../../entities/Message.entity");
describe('MessageService', () => {
    let messageService;
    let dataSource;
    beforeEach(() => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        // Создаем тестовую базу данных в памяти
        dataSource = new typeorm_1.DataSource({
            type: 'sqlite',
            database: ':memory:',
            entities: [Message_entity_1.MessageEntity],
            synchronize: true,
        });
        yield dataSource.initialize();
        messageService = new Message_service_1.MessageService(dataSource);
    }));
    afterEach(() => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        yield dataSource.destroy();
    }));
    describe('addMessage', () => {
        it('должен сохранять сообщение с payload', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            const testPayload = { test: 'data' };
            const message = {
                message_id: '123',
                chat_id: 456,
                user_id: 789,
                payload: testPayload,
            };
            yield messageService.addMessage(message);
            const savedMessage = yield dataSource.getRepository(Message_entity_1.MessageEntity).findOne({ where: { message_id: '123' } });
            expect(savedMessage).toBeDefined();
            expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload).toEqual(testPayload);
        }));
        it('должен сохранять не более 10 сообщений без payload в чате', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            // Создаем 12 сообщений без payload
            for (let i = 0; i < 12; i++) {
                yield messageService.addMessage({
                    message_id: i.toString(),
                    chat_id: 456,
                    user_id: 789,
                });
            }
            const messages = yield dataSource.getRepository(Message_entity_1.MessageEntity).find({
                where: { chat_id: 456 },
                order: { created_at: 'DESC' },
            });
            expect(messages.length).toBe(10);
        }));
        it('не должен удалять сообщения с payload при превышении лимита', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            // Создаем 5 сообщений с payload
            for (let i = 0; i < 5; i++) {
                yield messageService.addMessage({
                    message_id: `payload_${i}`,
                    chat_id: 456,
                    user_id: 789,
                    payload: { data: i },
                });
            }
            // Создаем 12 сообщений без payload
            for (let i = 0; i < 12; i++) {
                yield messageService.addMessage({
                    message_id: `no_payload_${i}`,
                    chat_id: 456,
                    user_id: 789,
                });
            }
            const allMessages = yield dataSource.getRepository(Message_entity_1.MessageEntity).find({
                where: { chat_id: 456 },
                order: { created_at: 'DESC' },
            });
            const messagesWithPayload = allMessages.filter((msg) => msg.payload !== null);
            const messagesWithoutPayload = allMessages.filter((msg) => msg.payload === null);
            expect(messagesWithPayload.length).toBe(5); // Все сообщения с payload должны остаться
            expect(messagesWithoutPayload.length).toBe(10); // Только 10 последних сообщений без payload
        }));
        describe('надежность сохранения payload', () => {
            it('должен корректно сохранять сложные вложенные объекты в payload', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const complexPayload = {
                    level1: {
                        level2: {
                            level3: {
                                array: [1, 2, 3],
                                string: 'test',
                                number: 123,
                                boolean: true,
                                null: null,
                            },
                        },
                        array: [{ a: 1 }, { b: 2 }],
                    },
                };
                yield messageService.addMessage({
                    message_id: 'complex_1',
                    chat_id: 456,
                    payload: complexPayload,
                });
                const savedMessage = yield dataSource.getRepository(Message_entity_1.MessageEntity).findOne({ where: { message_id: 'complex_1' } });
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload).toEqual(complexPayload);
            }));
            it('должен корректно сохранять массивы различных типов в payload', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const arrayPayload = {
                    numbers: [1, 2, 3, 4, 5],
                    strings: ['a', 'b', 'c'],
                    mixed: [1, 'two', { three: 3 }, [4]],
                    empty: [],
                };
                yield messageService.addMessage({
                    message_id: 'array_1',
                    chat_id: 456,
                    payload: arrayPayload,
                });
                const savedMessage = yield dataSource.getRepository(Message_entity_1.MessageEntity).findOne({ where: { message_id: 'array_1' } });
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload).toEqual(arrayPayload);
            }));
            it('должен корректно сохранять специальные значения в payload', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const specialPayload = {
                    maxNumber: Number.MAX_SAFE_INTEGER,
                    minNumber: Number.MIN_SAFE_INTEGER,
                    infinity: Infinity,
                    negativeInfinity: -Infinity,
                    nan: NaN,
                    undefined: undefined,
                    nullValue: null,
                    emptyString: '',
                    zero: 0,
                };
                yield messageService.addMessage({
                    message_id: 'special_1',
                    chat_id: 456,
                    payload: specialPayload,
                });
                const savedMessage = yield dataSource.getRepository(Message_entity_1.MessageEntity).findOne({ where: { message_id: 'special_1' } });
                // Проверяем каждое значение отдельно из-за особенностей сериализации
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.maxNumber).toBe(Number.MAX_SAFE_INTEGER);
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.minNumber).toBe(Number.MIN_SAFE_INTEGER);
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.nullValue).toBeNull();
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.emptyString).toBe('');
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.zero).toBe(0);
                // Infinity, NaN и undefined преобразуются в null при JSON сериализации
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.infinity).toBeNull();
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.negativeInfinity).toBeNull();
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.nan).toBeNull();
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload.undefined).toBeUndefined();
            }));
            it('должен корректно обрабатывать обновление payload существующего сообщения', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const initialPayload = { data: 'initial' };
                const updatedPayload = { data: 'updated' };
                // Создаем сообщение
                yield messageService.addMessage({
                    message_id: 'update_1',
                    chat_id: 456,
                    payload: initialPayload,
                });
                // Обновляем payload
                yield messageService.addMessage({
                    message_id: 'update_1',
                    chat_id: 456,
                    payload: updatedPayload,
                });
                const savedMessage = yield dataSource.getRepository(Message_entity_1.MessageEntity).findOne({ where: { message_id: 'update_1' } });
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload).toEqual(updatedPayload);
            }));
            it('должен сохранять большие объекты в payload', () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const largePayload = {
                    data: Array(1000)
                        .fill(0)
                        .map((_, i) => ({
                        index: i,
                        value: `value_${i}`,
                        nested: {
                            data: `nested_${i}`,
                            array: Array(10).fill(i),
                        },
                    })),
                };
                yield messageService.addMessage({
                    message_id: 'large_1',
                    chat_id: 456,
                    payload: largePayload,
                });
                const savedMessage = yield dataSource.getRepository(Message_entity_1.MessageEntity).findOne({ where: { message_id: 'large_1' } });
                expect(savedMessage === null || savedMessage === void 0 ? void 0 : savedMessage.payload).toEqual(largePayload);
            }));
        });
    });
});
