"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyboardBuilder = void 0;
const KeyboardConvert_1 = require("./KeyboardConvert");
class KeyboardBuilder {
    constructor(client) {
        this.client = client;
        this.keyboard = [];
        this.buttons = []; // Store raw button data
    }
    /** Adds a button to the keyboard */
    addButton(button) {
        this.buttons.push(button);
        return this;
    }
    /** Sets the order of buttons based on the provided array */
    setButtonOrder(order) {
        this.keyboard = []; // Clear the current keyboard layout
        let index = 0;
        for (const count of order) {
            const currentRow = [];
            for (let i = 0; i < count; i++) {
                if (index < this.buttons.length) {
                    currentRow.push(this.buttons[index++]); // Push buttons according to the specified order
                }
            }
            this.keyboard.push(currentRow); // Add the current row to the keyboard
        }
        return this; // Enable method chaining
    }
    /** Arranges buttons into a keyboard layout with pagination */
    arrangeButtons(buttons, width, height, command, page = 1) {
        this.keyboard = []; // Clear existing keyboard layout
        this.buttons = buttons; // Update the stored buttons
        const totalButtons = buttons.length;
        const buttonsPerPage = width * height; // Total buttons per page
        const pages = Math.ceil(totalButtons / buttonsPerPage); // Calculate total pages
        // Ensure the requested page number is valid
        if (page < 1 || page > pages) {
            throw new Error(`Page number ${page} is out of range. There are only ${pages} pages.`);
        }
        const start = (page - 1) * buttonsPerPage; // Starting index for the requested page
        const end = Math.min(start + buttonsPerPage, totalButtons); // Ending index for the requested page
        // If there are no more buttons to add, return
        if (start >= totalButtons)
            return this;
        const currentPageButtons = buttons.slice(start, end);
        const rows = this.createRows(currentPageButtons, width); // Create rows for the requested page
        // Add rows to the keyboard while respecting the height limit
        for (let rowIndex = 0; rowIndex < rows.length && rowIndex < height; rowIndex++) {
            this.keyboard.push(rows[rowIndex]); // Add only up to the specified height
        }
        // Add pagination buttons if there are more pages available
        this.addPaginationButtons(page, pages, command);
        return this;
    }
    /** Creates rows based on button configurations and specified width */
    createRows(buttonConfigs, width) {
        const rows = [];
        for (let i = 0; i < buttonConfigs.length; i += width) {
            const row = buttonConfigs.slice(i, i + width);
            rows.push(row);
        }
        return rows;
    }
    /** Adds pagination buttons for navigating between pages */
    addPaginationButtons(currentPage, totalPages, command) {
        const paginationButtons = [];
        if (currentPage > 1) {
            // Add "Previous" button if not on the first page
            paginationButtons.push({
                text: '« Previous',
                command, // Command for navigating to the previous page
                payload: { page: currentPage - 1 },
            });
        }
        if (currentPage < totalPages) {
            // Add "Next" button if not on the last page
            paginationButtons.push({
                text: 'Next »',
                command, // Command for navigating to the next page
                payload: { page: currentPage + 1 },
            });
        }
        if (paginationButtons.length > 0) {
            this.keyboard.push(paginationButtons); // Add pagination buttons as a new row
        }
    }
    /** Builds and returns the keyboard */
    build() {
        if (this.buttons.length > 0 && this.keyboard.length === 0) {
            this.setButtonOrder([this.buttons.length]);
        }
        return (0, KeyboardConvert_1.KeyboardConvert)(this.client, this.keyboard);
    }
}
exports.KeyboardBuilder = KeyboardBuilder;
