"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = exports.LogLevel = void 0;
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const util_1 = tslib_1.__importDefault(require("util"));
var LogLevel;
(function (LogLevel) {
    LogLevel["ERROR"] = "error";
    LogLevel["WARN"] = "warn";
    LogLevel["LOG"] = "log";
    LogLevel["DEBUG"] = "debug";
    LogLevel["VERBOSE"] = "verbose";
})(LogLevel || (exports.LogLevel = LogLevel = {}));
const logLevelPriority = {
    [LogLevel.ERROR]: 0,
    [LogLevel.WARN]: 1,
    [LogLevel.LOG]: 2,
    [LogLevel.DEBUG]: 3,
    [LogLevel.VERBOSE]: 4,
};
class Logger {
    constructor(context = 'Evogram', options = {}) {
        this.context = context;
        this.logLevel = options.logLevel || LogLevel.LOG;
        this.colorsEnabled = options.colors !== undefined ? options.colors : true;
    }
    getLogLevelPriority(level) {
        return logLevelPriority[level] || logLevelPriority[LogLevel.LOG];
    }
    getTimestamp() {
        return new Date().toLocaleString('en-US', {
            month: '2-digit',
            day: '2-digit',
            year: 'numeric',
            hour: '2-digit',
            minute: '2-digit',
            second: '2-digit',
            hour12: true,
        });
    }
    formatMessage(level, message, context, stack) {
        const timestamp = this.getTimestamp();
        const pid = process.pid;
        const color = this.getColor(level);
        let logMessage = `${color(`[Evogram] ${pid.toString().padEnd(6)} -`)} ${timestamp} ${color(level.toUpperCase().padStart(7))} ${chalk_1.default.yellow(`[${context}]`)} ${color(message)}`;
        if (stack) {
            const coloredStack = this.colorsEnabled ? chalk_1.default.red(stack) : stack;
            logMessage += `\n${coloredStack}`;
        }
        return logMessage;
    }
    getColor(level) {
        if (!this.colorsEnabled)
            return (text) => text;
        switch (level) {
            case LogLevel.ERROR:
                return chalk_1.default.red;
            case LogLevel.WARN:
                return chalk_1.default.yellow;
            case LogLevel.LOG:
                return chalk_1.default.green;
            case LogLevel.DEBUG:
                return chalk_1.default.blue;
            case LogLevel.VERBOSE:
                return chalk_1.default.gray;
            default:
                return chalk_1.default.white;
        }
    }
    printMessage(level, message, context, stack) {
        if (this.getLogLevelPriority(level) > this.getLogLevelPriority(this.logLevel))
            return;
        if (typeof message !== 'string') {
            message = util_1.default.inspect(message, {
                colors: this.colorsEnabled,
                depth: null,
            });
        }
        const formattedMessage = this.formatMessage(level, message, context, stack);
        // prettier-ignore
        // @ts-ignore
        console[{ [LogLevel.DEBUG]: 'debug', [LogLevel.ERROR]: 'error', [LogLevel.WARN]: 'warn' }[level] || 'log'](formattedMessage);
    }
    error(message, trace, context) {
        this.printMessage(LogLevel.ERROR, message, context || this.context, trace);
    }
    warn(message, context) {
        this.printMessage(LogLevel.WARN, message, context || this.context);
    }
    log(message, context) {
        this.printMessage(LogLevel.LOG, message, context || this.context);
    }
    debug(message, context) {
        this.printMessage(LogLevel.DEBUG, message, context || this.context);
    }
    verbose(message, context) {
        this.printMessage(LogLevel.VERBOSE, message, context || this.context);
    }
    child(context) {
        return new Logger(context, {
            logLevel: this.logLevel,
            colors: this.colorsEnabled,
        });
    }
    static create(context) {
        return new Logger(context);
    }
}
exports.Logger = Logger;
