"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Middleware = void 0;
const tslib_1 = require("tslib");
const chatDB_middleware_1 = require("./middlewares/chatDB.middleware");
const floodControl_middleware_1 = require("./middlewares/floodControl.middleware");
const userDB_middleware_1 = require("./middlewares/userDB.middleware");
/**
 * Manages the registration and execution of middleware functions for the Evogram client.
 */
class Middleware {
    constructor() {
        // Array to hold registered middleware functions
        this.middlewares = [];
        this.middlewares.push(floodControl_middleware_1.FloodControlMiddleware.middleware);
        this.middlewares.push(userDB_middleware_1.UserDBMiddleware.middleware);
        this.middlewares.push(chatDB_middleware_1.ChatDBMiddleware.middleware);
    }
    /**
     * Registers a middleware function to the stack.
     * @param middleware The middleware function to register.
     */
    use(middleware) {
        this.middlewares.push(middleware);
    }
    /**
     * Executes all registered middleware functions in order.
     * @param context The context to pass to middleware (includes TelegramUpdate and client).
     * @returns The context if all middleware executed successfully, or null if not.
     */
    execute(context) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let allNextCalled = true;
            const middlewares = [...this.middlewares, ...Middleware.middlewares];
            // Recursive function to run middleware in order
            const run = (index) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (index < middlewares.length) {
                    const middleware = middlewares[index];
                    let nextCalled = false;
                    try {
                        // Call the current middleware, providing context and the next function
                        yield middleware(context, () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                            nextCalled = true;
                            yield run(index + 1); // Run the next middleware
                        }));
                    }
                    catch (error) {
                        console.error(`Error in middleware at index ${index}:`, error);
                        allNextCalled = false; // Middleware execution not fully successful
                    }
                    // Check if 'next' was called in this middleware
                    if (!nextCalled)
                        allNextCalled = false; // If 'next' wasn't called, mark as incomplete
                }
            });
            // Start running from the first middleware
            yield run(0);
            return allNextCalled ? context : null; // Return context if all middlewares executed successfully
        });
    }
}
exports.Middleware = Middleware;
Middleware.middlewares = [];
