"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Polling = void 0;
const tslib_1 = require("tslib");
const EventTransport_1 = require("./EventTransport");
/**
 * Represents a class for handling long-polling updates from the Telegram Bot API.
 */
class Polling extends EventTransport_1.EventTransport {
    constructor() {
        super(...arguments);
        this.lastUpdateTime = Date.now();
    }
    /**
     * Starts long-polling with optional parameters.
     * @param {TelegramGetUpdatesParams} [params] - Optional parameters for long-polling.
     */
    start(params) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            console.log(`\x1b[34m❯\x1b[0m Bot has been started! (Polling)`);
            yield this.client.database.init();
            console.log(`\x1b[34m❯\x1b[0m Database connected!`);
            this.state = EventTransport_1.EventTransportState.Enabled;
            yield this.worker(params);
        });
    }
    /**
     * Stops long-polling.
     */
    stop() {
        console.log(`\x1b[31m❯\x1b[0m Bot has been stopped! (Polling)`);
        this.state = EventTransport_1.EventTransportState.Disabled;
    }
    /**
     * Worker function for long-polling updates.
     * @param {TelegramGetUpdatesParams} [params={}] - Optional parameters for updates.
     */
    worker() {
        return tslib_1.__awaiter(this, arguments, void 0, function* (params = {}) {
            while (this.state === EventTransport_1.EventTransportState.Enabled) {
                try {
                    if (Date.now() - this.lastUpdateTime > 60000) {
                        this.lastUpdateTime = Date.now();
                        this.worker(params);
                        return console.log('⏳ Restarting worker due to inactivity...');
                    }
                    else
                        this.lastUpdateTime = Date.now();
                    // Fetch updates from the Telegram API
                    const updates = yield this.client.api.getUpdates(params);
                    if (!updates || updates.length === 0)
                        continue;
                    // Process each update
                    for (const update of updates) {
                        this.onUpdate(update);
                        // Update the offset parameter to fetch only new updates next time
                        params.offset = update.id + 1;
                    }
                }
                catch (error) {
                    console.error(`\x1b[31m❌\x1b[0m Error during polling:`, error);
                }
            }
        });
    }
}
exports.Polling = Polling;
