"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomTagManager = void 0;
const tslib_1 = require("tslib");
class CustomTagManager {
    // Регистрация нового пользовательского тега
    static register(tag, callback) {
        this.tags.set(tag.toLowerCase(), { callback });
    }
    // Асинхронная обработка текста с пользовательскими тегами
    static process(client, text) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const self = this;
            // Универсальный парсер одиночных и парных тегов (без привязки к зарегистрированным)
            function parseNodes(input) {
                const nodes = [];
                const tagOpenRe = /^<([a-zA-Z0-9_\-]+)(\s+[^>]*)?>/;
                const tagSelfCloseRe = /^<([a-zA-Z0-9_\-]+)(\s+[^>]*)?\s*\/?>/;
                const tagCloseRe = (name) => new RegExp(`^</${name}>`);
                let i = 0;
                const len = input.length;
                const stack = [];
                let buffer = '';
                while (i < len) {
                    if (input[i] === '<') {
                        const rest = input.slice(i);
                        let m = null;
                        // Одиночный тег (только если есть / перед >)
                        if ((m = /^<([a-zA-Z0-9_\-]+)(\s+[^>]*)?\s*\/>/.exec(rest))) {
                            if (buffer) {
                                (stack.length ? stack[stack.length - 1].children : nodes).push({ type: 'text', value: buffer });
                                buffer = '';
                            }
                            const tagName = m[1].toLowerCase();
                            const attrs = {};
                            const attrsStr = m[2] ? m[2].trim() : '';
                            attrsStr.replace(/([a-zA-Z0-9_\-]+)="([^"]*)"/g, (_, k, v) => ((attrs[k] = v), ''));
                            const node = { type: 'tag', name: tagName, attrs, children: [] };
                            (stack.length ? stack[stack.length - 1].children : nodes).push(node);
                            i += m[0].length;
                            continue;
                        }
                        // Открывающий тег (только если НЕТ / перед >)
                        if ((m = /^<([a-zA-Z0-9_\-]+)(\s+[^>]*)?>/.exec(rest))) {
                            // Проверяем, что это не одиночный тег (нет / перед >)
                            const isSelfClosing = /\/>$/.test(m[0]);
                            if (!isSelfClosing) {
                                if (buffer) {
                                    (stack.length ? stack[stack.length - 1].children : nodes).push({ type: 'text', value: buffer });
                                    buffer = '';
                                }
                                const tagName = m[1].toLowerCase();
                                const attrs = {};
                                const attrsStr = m[2] ? m[2].trim() : '';
                                attrsStr.replace(/([a-zA-Z0-9_\-]+)="([^"]*)"/g, (_, k, v) => ((attrs[k] = v), ''));
                                stack.push({ name: tagName, attrs, children: [] });
                                i += m[0].length;
                                continue;
                            }
                        }
                        // Закрывающий тег
                        if (stack.length && tagCloseRe(stack[stack.length - 1].name).test(rest)) {
                            if (buffer) {
                                stack[stack.length - 1].children.push({ type: 'text', value: buffer });
                                buffer = '';
                            }
                            const node = stack.pop();
                            const tagNode = { type: 'tag', name: node.name, attrs: node.attrs, children: node.children };
                            if (stack.length) {
                                stack[stack.length - 1].children.push(tagNode);
                            }
                            else {
                                nodes.push(tagNode);
                            }
                            i += `</${node.name}>`.length;
                            continue;
                        }
                    }
                    // Просто текст
                    buffer += input[i];
                    i++;
                }
                if (buffer) {
                    (stack.length ? stack[stack.length - 1].children : nodes).push({ type: 'text', value: buffer });
                }
                // Если остались незакрытые теги — считаем их текстом
                while (stack.length) {
                    const node = stack.pop();
                    const tagNode = { type: 'tag', name: node.name, attrs: node.attrs, children: node.children };
                    nodes.push(tagNode);
                }
                return nodes;
            }
            // Рекурсивная обработка дерева узлов
            const processNodes = (nodes) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                let out = '';
                for (const node of nodes) {
                    if (node.type === 'text') {
                        out += node.value;
                    }
                    else {
                        const def = self.tags.get(node.name.toLowerCase());
                        const inner = node.children.length ? yield processNodes(node.children) : undefined;
                        if (!def) {
                            const attrsStr = Object.entries(node.attrs)
                                .map(([k, v]) => `${k}="${v}"`)
                                .join(' ');
                            if (node.children.length) {
                                out += `<${node.name}${attrsStr ? ' ' + attrsStr : ''}>${inner}</${node.name}>`;
                            }
                            else {
                                out += `<${node.name}${attrsStr ? ' ' + attrsStr : ''} />`;
                            }
                        }
                        else {
                            let result = yield def.callback(client, node.attrs, inner);
                            // Если результат содержит кастомные теги — прогоняем через process ещё раз
                            const tagNames = Array.from(self.tags.keys());
                            if (tagNames.length > 0) {
                                const tagGroup = tagNames.map((t) => String(t).replace(/[-/\\^$*+?.()|[\]{}]/g, '\\$&')).join('|');
                                const customTagRe = new RegExp(`<(${tagGroup})([\s>])`, 'i');
                                if (customTagRe.test(result)) {
                                    result = yield self.process(client, result);
                                }
                            }
                            out += result;
                        }
                    }
                }
                return out;
            });
            const nodes = parseNodes(text);
            return processNodes(nodes);
        });
    }
}
exports.CustomTagManager = CustomTagManager;
// Хранилище зарегистрированных тегов: имя -> описание
CustomTagManager.tags = new Map();
