"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emailValidator = exports.EMAIL_REGEX = exports.disposableDomains = exports.EmailValidatorError = void 0;
const tslib_1 = require("tslib");
const errors_1 = require("./errors");
const dns_1 = tslib_1.__importDefault(require("dns"));
const util_1 = require("util");
const resolveMx = (0, util_1.promisify)(dns_1.default.resolveMx);
var EmailValidatorError;
(function (EmailValidatorError) {
    EmailValidatorError["VALUE_REQUIRED"] = "VALUE_REQUIRED";
    EmailValidatorError["INVALID_FORMAT"] = "INVALID_FORMAT";
    EmailValidatorError["INVALID_DOMAIN"] = "INVALID_DOMAIN";
    EmailValidatorError["DISPOSABLE_EMAIL"] = "DISPOSABLE_EMAIL";
    EmailValidatorError["BLOCKED_DOMAIN"] = "BLOCKED_DOMAIN";
    EmailValidatorError["MX_RECORD_NOT_FOUND"] = "MX_RECORD_NOT_FOUND";
})(EmailValidatorError || (exports.EmailValidatorError = EmailValidatorError = {}));
exports.disposableDomains = new Set([
    'tempmail.com',
    'throwawaymail.com',
    '10minutemail.com',
    'guerrillamail.com',
    'mailinator.com',
    'yopmail.com',
    'dispostable.com',
    'maildrop.cc',
    'temp-mail.org',
    'fakeinbox.com',
    'sharklasers.com',
    'trashmail.com',
    'wegwerfmail.de',
    'tempmail.net',
    'emailondeck.com',
    'tempinbox.com',
    'mohmal.com',
    'getnada.com',
    'tempmailaddress.com',
    'mintemail.com',
    'tempmail.ninja',
    'disposablemail.com',
]);
exports.EMAIL_REGEX = /^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(?:\.[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/;
const emailValidator = (options = {}) => {
    return (context) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const { value, context: cmdContext, arg } = context;
        if (!value && options.question) {
            yield options.question(cmdContext);
            return undefined;
        }
        if (!value) {
            if (!arg.isOptional)
                throw new errors_1.ValidatorError(options, EmailValidatorError.VALUE_REQUIRED, errors_1.ValidationErrors.get(EmailValidatorError.VALUE_REQUIRED) || `Email is required`);
            else
                return;
        }
        const email = value.trim().toLowerCase();
        if (!exports.EMAIL_REGEX.test(email)) {
            throw new errors_1.ValidatorError(options, EmailValidatorError.INVALID_FORMAT, errors_1.ValidationErrors.get(EmailValidatorError.INVALID_FORMAT) || `Email «${arg.name}» is invalid`);
        }
        const [, domain] = email.split('@');
        if (((_a = options.allowedDomains) === null || _a === void 0 ? void 0 : _a.length) && !options.allowedDomains.includes(domain)) {
            throw new errors_1.ValidatorError(options, EmailValidatorError.INVALID_DOMAIN, errors_1.ValidationErrors.get(EmailValidatorError.INVALID_DOMAIN) || `Email «${arg.name}» is invalid`);
        }
        if (((_b = options.blockedDomains) === null || _b === void 0 ? void 0 : _b.length) && options.blockedDomains.includes(domain)) {
            throw new errors_1.ValidatorError(options, EmailValidatorError.BLOCKED_DOMAIN, errors_1.ValidationErrors.get(EmailValidatorError.BLOCKED_DOMAIN) || `Email «${arg.name}» is invalid`);
        }
        if (options.checkDisposable && exports.disposableDomains.has(domain)) {
            throw new errors_1.ValidatorError(options, EmailValidatorError.DISPOSABLE_EMAIL, errors_1.ValidationErrors.get(EmailValidatorError.DISPOSABLE_EMAIL) || `Email «${arg.name}» is disposable`);
        }
        if (options.checkMx) {
            try {
                const mxRecords = yield resolveMx(domain);
                if (mxRecords.length === 0) {
                    throw new errors_1.ValidatorError(options, EmailValidatorError.MX_RECORD_NOT_FOUND, errors_1.ValidationErrors.get(EmailValidatorError.MX_RECORD_NOT_FOUND) || `Email «${arg.name}» is invalid`);
                }
            }
            catch (_c) {
                throw new errors_1.ValidatorError(options, EmailValidatorError.MX_RECORD_NOT_FOUND, errors_1.ValidationErrors.get(EmailValidatorError.MX_RECORD_NOT_FOUND) || `Email «${arg.name}» is invalid`);
            }
        }
        return email;
    });
};
exports.emailValidator = emailValidator;
