"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.phoneValidator = exports.formatPhone = exports.normalizePhone = exports.COUNTRY_PATTERNS = exports.PhoneValidatorError = void 0;
const tslib_1 = require("tslib");
const errors_1 = require("./errors");
var PhoneValidatorError;
(function (PhoneValidatorError) {
    PhoneValidatorError["VALUE_REQUIRED"] = "VALUE_REQUIRED";
    PhoneValidatorError["INVALID_FORMAT"] = "INVALID_FORMAT";
    PhoneValidatorError["INVALID_COUNTRY_CODE"] = "INVALID_COUNTRY_CODE";
    PhoneValidatorError["INVALID_LENGTH"] = "INVALID_LENGTH";
    PhoneValidatorError["INVALID_OPERATOR"] = "INVALID_OPERATOR";
})(PhoneValidatorError || (exports.PhoneValidatorError = PhoneValidatorError = {}));
exports.COUNTRY_PATTERNS = {
    RU: {
        pattern: /^(\+7|7|8)?[\s-]?\(?[489][0-9]{2}\)?[\s-]?[0-9]{3}[\s-]?[0-9]{2}[\s-]?[0-9]{2}$/,
        length: 11,
    },
    US: {
        pattern: /^(\+1|1)?[\s-]?\(?[0-9]{3}\)?[\s-]?[0-9]{3}[\s-]?[0-9]{4}$/,
        length: 10,
    },
    UA: {
        pattern: /^(\+380|380|0)?[\s-]?\(?[0-9]{2}\)?[\s-]?[0-9]{3}[\s-]?[0-9]{2}[\s-]?[0-9]{2}$/,
        length: 12,
    },
    BY: {
        pattern: /^(\+375|375)?[\s-]?\(?[0-9]{2}\)?[\s-]?[0-9]{3}[\s-]?[0-9]{2}[\s-]?[0-9]{2}$/,
        length: 12,
    },
    KZ: {
        pattern: /^(\+7|7)?[\s-]?\(?[0-9]{3}\)?[\s-]?[0-9]{3}[\s-]?[0-9]{2}[\s-]?[0-9]{2}$/,
        length: 11,
    },
    DE: {
        pattern: /^(\+49|49)?[\s-]?\(?[0-9]{3,5}\)?[\s-]?[0-9]{3,4}[\s-]?[0-9]{2,4}$/,
        length: 11,
    },
    FR: {
        pattern: /^(\+33|33)?[\s-]?\(?[0-9]{1}\)?[\s-]?[0-9]{2}[\s-]?[0-9]{2}[\s-]?[0-9]{2}[\s-]?[0-9]{2}$/,
        length: 11,
    },
    GB: {
        pattern: /^(\+44|44)?[\s-]?\(?[0-9]{2}\)?[\s-]?[0-9]{4}[\s-]?[0-9]{4}$/,
        length: 11,
    },
    CN: {
        pattern: /^(\+86|86)?[\s-]?\(?[0-9]{3}\)?[\s-]?[0-9]{4}[\s-]?[0-9]{4}$/,
        length: 11,
    },
    JP: {
        pattern: /^(\+81|81)?[\s-]?\(?[0-9]{2,3}\)?[\s-]?[0-9]{3,4}[\s-]?[0-9]{4}$/,
        length: 11,
    },
};
const normalizePhone = (phone) => {
    return phone.replace(/[\s\-\(\)+]/g, '');
};
exports.normalizePhone = normalizePhone;
const formatPhone = (phone, format) => {
    const digits = phone.replace(/\D/g, '');
    let result = format;
    let index = 0;
    return result.replace(/[#]/g, () => digits[index++] || '');
};
exports.formatPhone = formatPhone;
const phoneValidator = (options = {}) => {
    return (context) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const { value, context: cmdContext, arg } = context;
        if (!value && options.question) {
            yield options.question(cmdContext);
            return undefined;
        }
        if (!value) {
            if (!arg.isOptional)
                throw new errors_1.ValidatorError(options, PhoneValidatorError.VALUE_REQUIRED, errors_1.ValidationErrors.get(PhoneValidatorError.VALUE_REQUIRED) || `Phone «${arg.name}» is required`);
            else
                return { phone: 0, formatted: '' };
        }
        const normalizedPhone = (0, exports.normalizePhone)(value);
        if (options.country) {
            const countryPattern = exports.COUNTRY_PATTERNS[options.country];
            if (!countryPattern) {
                throw new errors_1.ValidatorError(options, PhoneValidatorError.INVALID_COUNTRY_CODE, errors_1.ValidationErrors.get(PhoneValidatorError.INVALID_COUNTRY_CODE) || `Invalid country code: ${options.country}`);
            }
            if (!countryPattern.pattern.test(value)) {
                throw new errors_1.ValidatorError(options, PhoneValidatorError.INVALID_FORMAT, errors_1.ValidationErrors.get(PhoneValidatorError.INVALID_FORMAT) || `Invalid phone format for country: ${options.country}`);
            }
            if (normalizedPhone.length !== countryPattern.length) {
                throw new errors_1.ValidatorError(options, PhoneValidatorError.INVALID_LENGTH, errors_1.ValidationErrors.get(PhoneValidatorError.INVALID_LENGTH) || `Invalid phone length for country: ${options.country}`);
            }
        }
        else {
            const generalPhonePattern = /^\+?[1-9]\d{1,14}$/; // Общий формат номера телефона
            if (!generalPhonePattern.test(normalizedPhone)) {
                throw new errors_1.ValidatorError(options, PhoneValidatorError.INVALID_FORMAT, errors_1.ValidationErrors.get(PhoneValidatorError.INVALID_FORMAT) || `Invalid phone format`);
            }
        }
        if ((_a = options.allowedCountries) === null || _a === void 0 ? void 0 : _a.length) {
            const isValidCountry = options.allowedCountries.some((country) => { var _a; return (_a = exports.COUNTRY_PATTERNS[country]) === null || _a === void 0 ? void 0 : _a.pattern.test(value); });
            if (!isValidCountry) {
                throw new errors_1.ValidatorError(options, PhoneValidatorError.INVALID_COUNTRY_CODE, errors_1.ValidationErrors.get(PhoneValidatorError.INVALID_COUNTRY_CODE) || `Invalid country code: ${options.country}`);
            }
        }
        if ((_b = options.allowedOperators) === null || _b === void 0 ? void 0 : _b.length) {
            const operatorCode = normalizedPhone.slice(1, 4);
            if (!options.allowedOperators.includes(operatorCode)) {
                throw new errors_1.ValidatorError(options, PhoneValidatorError.INVALID_OPERATOR, errors_1.ValidationErrors.get(PhoneValidatorError.INVALID_OPERATOR) || `Invalid operator code: ${operatorCode}`);
            }
        }
        let formattedPhone = normalizedPhone;
        if (normalizedPhone.length === 11) {
            formattedPhone = `+${normalizedPhone.slice(0, 1)} (${normalizedPhone.slice(1, 4)}) ${normalizedPhone.slice(4, 6)} ${normalizedPhone.slice(6, 8)} ${normalizedPhone.slice(8)}`;
        }
        else if (normalizedPhone.length === 12) {
            formattedPhone = `+${normalizedPhone.slice(0, 2)} (${normalizedPhone.slice(2, 5)}) ${normalizedPhone.slice(5, 7)} ${normalizedPhone.slice(7, 9)} ${normalizedPhone.slice(9)}`;
        }
        else if (normalizedPhone.length > 12) {
            formattedPhone = `+${normalizedPhone.slice(0, 1)} (${normalizedPhone.slice(1, 4)}) ${normalizedPhone.slice(4, 7)} ${normalizedPhone.slice(7, 9)} ${normalizedPhone.slice(9)}`;
        }
        if (options.formats) {
            const format = options.formats.find((format) => { var _a; return (_a = exports.COUNTRY_PATTERNS[format]) === null || _a === void 0 ? void 0 : _a.pattern.test(value); });
            if (!format) {
                throw new errors_1.ValidatorError(options, PhoneValidatorError.INVALID_FORMAT, errors_1.ValidationErrors.get(PhoneValidatorError.INVALID_FORMAT) || `Invalid phone format for country: ${options.country}`);
            }
            formattedPhone = (0, exports.formatPhone)(normalizedPhone, format);
        }
        return { phone: Number(normalizedPhone), formatted: formattedPhone };
    });
};
exports.phoneValidator = phoneValidator;
