"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.urlValidator = exports.isValidUrl = exports.UrlValidatorError = void 0;
const tslib_1 = require("tslib");
const errors_1 = require("./errors");
const axios_1 = tslib_1.__importDefault(require("axios"));
var UrlValidatorError;
(function (UrlValidatorError) {
    UrlValidatorError["VALUE_REQUIRED"] = "VALUE_REQUIRED";
    UrlValidatorError["INVALID_URL"] = "INVALID_URL";
    UrlValidatorError["INVALID_PROTOCOL"] = "INVALID_PROTOCOL";
    UrlValidatorError["INVALID_DOMAIN"] = "INVALID_DOMAIN";
    UrlValidatorError["INVALID_CONTENT_TYPE"] = "INVALID_CONTENT_TYPE";
    UrlValidatorError["REQUEST_FAILED"] = "REQUEST_FAILED";
})(UrlValidatorError || (exports.UrlValidatorError = UrlValidatorError = {}));
const isValidUrl = (url) => {
    try {
        new URL(url);
        return true;
    }
    catch (_a) {
        return false;
    }
};
exports.isValidUrl = isValidUrl;
const urlValidator = (options = {}) => {
    return (context) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const { value, context: cmdContext, arg } = context;
        // Если значение отсутствует и есть функция question
        if (!value && options.question) {
            yield options.question(cmdContext);
            return undefined;
        }
        if (!value) {
            if (!arg.isOptional)
                throw new errors_1.ValidatorError(options, UrlValidatorError.VALUE_REQUIRED, errors_1.ValidationErrors.get(UrlValidatorError.VALUE_REQUIRED) || `URL «${arg.name}» is required`);
            else
                return;
        }
        const urlString = value.trim();
        // Проверяем, является ли строка валидным URL
        if (!(0, exports.isValidUrl)(urlString)) {
            throw new errors_1.ValidatorError(options, UrlValidatorError.INVALID_URL, errors_1.ValidationErrors.get(UrlValidatorError.INVALID_URL) || `URL «${arg.name}» is invalid`);
        }
        const url = new URL(urlString);
        // Проверяем протокол, если указаны разрешенные протоколы
        if ((_a = options.protocols) === null || _a === void 0 ? void 0 : _a.length) {
            const protocol = url.protocol.replace(':', '');
            if (!options.protocols.includes(protocol)) {
                throw new errors_1.ValidatorError(options, UrlValidatorError.INVALID_PROTOCOL, errors_1.ValidationErrors.get(UrlValidatorError.INVALID_PROTOCOL) || `Invalid protocol: ${protocol}`);
            }
        }
        // Проверяем домен, если указаны разрешенные домены
        if ((_b = options.allowedDomains) === null || _b === void 0 ? void 0 : _b.length) {
            const domain = url.hostname;
            if (!options.allowedDomains.some((allowedDomain) => domain === allowedDomain || domain.endsWith(`.${allowedDomain}`))) {
                throw new errors_1.ValidatorError(options, UrlValidatorError.INVALID_DOMAIN, errors_1.ValidationErrors.get(UrlValidatorError.INVALID_DOMAIN) || `Invalid domain: ${domain}`);
            }
        }
        // Проверяем тип контента, если указан
        if (options.contentType) {
            try {
                const response = yield axios_1.default.head(urlString);
                const contentType = response.headers['content-type'];
                const allowedTypes = Array.isArray(options.contentType) ? options.contentType : [options.contentType];
                if (!allowedTypes.some((type) => contentType.includes(type))) {
                    throw new errors_1.ValidatorError(options, UrlValidatorError.INVALID_CONTENT_TYPE, errors_1.ValidationErrors.get(UrlValidatorError.INVALID_CONTENT_TYPE) || `Invalid content type: ${contentType}`);
                }
            }
            catch (error) {
                throw new errors_1.ValidatorError(options, UrlValidatorError.REQUEST_FAILED, errors_1.ValidationErrors.get(UrlValidatorError.REQUEST_FAILED) || `Request failed: ${error}`);
            }
        }
        return urlString;
    });
};
exports.urlValidator = urlValidator;
